<?php

/*
 * Set this to true if you have problems -- check the README file
 * Note:  This doesn't work all of the time (No idea why)
 *        Seems to be related to UW
 *
 *        IMAP session code taken from filters plugin
 *
 * 24.03.2003 jed
 * 01.06.2003 jed - undefined offset error in 'start_uw_folders' solved
 * 04.07.2005 jed - excluded dot-files from selectable folder list
 */
global $UseSeparateImapConnection;
$UseSeparateImapConnection = false;

// SquirrelMail required files
require_once(SM_PATH . 'functions/imap_general.php');

// initialize uw_folders plugin
function squirrelmail_plugin_init_uw_folders() {
    global $squirrelmail_plugin_hooks;

    $squirrelmail_plugin_hooks['sqimap_mailbox_list_additional']['uw_folders'] = 'start_uw_folders';
}

/**
 * This functions adds shared and public folders to the folder list
 *
 * input:  array of folders
 * output: modified array of folders
 */
function start_uw_folders($folder_ary) {

    global $mailbox, $imapServerAddress, $imapPort, $imap, $imap_general, $filters,
           $imap_stream, $imapConnection, $UseSeparateImapConnection, $AllowSpamFilters,
           $_SESSION, $_COOKIE, $imap_server_type;

    $username = $_SESSION['username'];
    $key = $_COOKIE['key'];
    $ssid = sqimap_session_id();

    // check if uw imap server
    if ($imap_server_type = 'uw') {

        /**
         * Detect if we have already connected to IMAP or not.
         * Also check if we are forced to use a separate IMAP connection
         */
        if ((!isset($imap_stream) && !isset($imapConnection)) || $UseSeparateImapConnection) {
            $stream = sqimap_login($username, $key, $imapServerAddress, $imapPort, 10);
            $previously_connected = false;
        }
        elseif (isset($imapConnection)) {
            $stream = $imapConnection;
            $previously_connected = true;
        }
        else {
            $previously_connected = true;
            $stream = $imap_stream;
        }

        // get list of shared folders
        $sharedfolders = uw_folders_get_shared_folders ($stream);

        for ($i = 0; $i < count($sharedfolders); $i++) {
            // check if shared mailboxes exist
            fputs ($stream, $ssid . " LIST \"$sharedfolders[$i]\" *\r\n");
            $folder_ary_tmp = sqimap_read_data ($stream, $ssid, true, $response, $message);

            if (count($folder_ary_tmp) > 0) {
                // add shared mailbox to mailbox list
                array_push($folder_ary, $folder_ary_tmp[0]);
            }
        }

        if (!$previously_connected) {
            sqimap_logout($stream);
        }
    }

    return $folder_ary;
}

/**
 * This functions checks if shared and public folders exist on an
 * IMAP server. It has been tested with 'uw' but may also run with
 * other servers.
 *
 * input:  IMAP stream
 * output: array of shared/public folders
 */
function uw_folders_get_shared_folders ($stream) {

    global $sharedfolders;
    $sharedfolders = array();

    // check if IMAP server supports NAMESPACE command
    if (sqimap_capability($stream, 'NAMESPACE')) {

        // run NAMESPACE command
        $read = sqimap_run_command($stream, 'NAMESPACE', true, $a, $b);

        /**
         * get third parameter -> shared folders
         *
         * Request : A003 NAMESPACE  
         * Response: * NAMESPACE (("" "/")("#mhinbox" NIL)("#mh/" "/")) (("~" "/")) (("#shared/" "/")
         *                        ("#ftp/" "/")("#news." ".")("#public/" "/")) 
         *           NAMESPACE completed 
         */
        if (eregi('\\* NAMESPACE +(\\( *\\(.+\\) *\\)|NIL) +(\\( *\\(.+\\) *\\)|NIL) +(\\( *\\(.+\\) *\\)|NIL)', $read[0], $data1)) {
            // get list of shared folder names
            if (eregi('"(.*)".*".*".*"(.*)".*".*".*"(.*)".*".*".*"(.*)".*".*"', $data1[3], $data2)) {

                // check for existing folders
                for ($i = 1; $i < count($data2); $i++) {
                    if ($data2[$i]) {

                        // add parent folder
                        if (eregi('#(.*)', $data2[$i], $data3)) {
                            array_push($sharedfolders, chop($data3[0]));
                        }

                        /**
                         * check for mailboxes
                         *
                         * Request : A003 LIST "#shared/" "*"  
                         * Response: * LIST (\NoSelect) "/" #shared/ 
                         *           * LIST (\NoInferiors \UnMarked) "/" #shared/Family-News 
                         *           * LIST (\NoInferiors \UnMarked) "/" #shared/Soccer-News 
                         *           * LIST (\NoInferiors \UnMarked) "/" #shared/Theatre-News 
                         *           LIST completed
                         */
                        fputs ($stream, ". LIST \"$data2[$i]\" \"*\"\n");
                        $read = sqimap_read_data($stream, '.', true, $a, $b);

                        for ($j = 1; $j < count($read); $j++) {
                            if ($read[$j]) {

                                /**
                                 * add mailbox folder
                                 *
                                 * if folder begins with '#'
                                 *   and not contains '#public/.' and not contains '#shared/.' and not contains '#ftp/.'
                                 *   then add it!
                                 */
                                if (eregi('#(.*)', $read[$j], $data4) && !eregi('#public\/\.', $read[$j], $data4) && !eregi('#shared\/\.', $read[$j], $data4) && !eregi('#ftp\/\.', $read[$j], $data4)) {
                                // if (eregi('#(.*)\/\.!.*', $read[$j], $data4)) {
                                    array_push($sharedfolders, chop($data4[0]));
                                }
                            }
                        }

                    }
                }
            }
        }
    }
    return $sharedfolders;
}

?>
